#Online supplement

#Multi-year dynamics of single-step genomic prediction in an applied wheat breeding program

#Sebastian Michel1, Franziska Lschenberger2, Ellen Sparry3, Christian Ametz2, Hermann Buerstmayr1


#1Department for Agrobiotechnology (IFA-Tulln), Institute for Biotechnology in Plant Production, University of Natural Resources and Life Sciences, Vienna (BOKU), Konrad-Lorenz-Str. 20, 3430 Tulln, Austria
#2Saatzucht Donau GesmbH. & CoKG, Saatzuchtstrasse 11, 2301 Probstdorf, Austria
#3C&M Seeds, 6180 5th Line, Palmerston, ON N0G 2P0, Canada

#Corresponding author: S. Michel; e-mail: sebastian.michel@boku.ac.at
#####################################################################################################################
#Analysis of an example dataset for illustrating the utilized models

#Date: 07/08/2020
#Author: Sebastian Michel
#For questions/suggestions/comments contact: sebastian.michel@boku.ac.at


#The example dataset contains:
#>400 lines from 200 simulated families for training and validating the models described in the manuscript
#>2000 SNP markers representing linked loci for a quantitatively inherited traits
#>The data contains a column for the true genotypic value (tgv) and estimated genotypic value (egv)
# which was simulated by aiming at a heritability of h = 0.3 for the 3 years of multi-environment trials (MET)
# and a simulated heritabilty of h = 0.1 for the selection candidates in the preliminary yield trial (PYT)

#The trait of interest was simulated using 100 SNP markers as causal loci


###################
#Import the simulated dataset and the pedigree relationship matrix A
setwd("E:/PhD/Pubilications/Combined marker pedigree prediction/Example dataset SingleStep")
mydata <- read.table("data_wheat.csv",header=T,sep=";",dec=".",blank.lines.skip = TRUE)
A_matrix <-  read.table("A_wheat.csv",header=T,sep=";",dec=".",blank.lines.skip = TRUE)
A_matrix <- as.matrix(A_matrix)
colnames(A_matrix) <- rownames(A_matrix) <- 1:nrow(A_matrix)

mydata$LINE <- as.factor(as.character(mydata$LINE))
mydata$YEAR <- as.factor(mydata$YEAR)
mydata$GROUP <- as.factor(mydata$GROUP)

str(mydata[,1:10])


#Seperate the training and validation population
data_train <- droplevels(mydata[!(mydata$YEAR %in% "PYT/2018"),])
data_valid <- droplevels(mydata[which(mydata$YEAR %in% "PYT/2018"),])



###################
#Fit a G-BLUP model without/and with pre-existing information of the selection candidates
data_model <- rbind(data_train,data_valid)
G_matrix <- A.mat(as.matrix(data_model[,-c(1:5)]),min.MAF=0.05)
rownames(G_matrix) <- colnames(G_matrix) <- data_model$LINE

data_model$egv[which(data_model$LINE %in% data_valid$LINE)] <- NA
model <- mmer(fixed=egv~YEAR,random=~vs(LINE,Gu=G_matrix),data=data_model,verbose=F,date.warning = F)
gblup <- data.frame(LINE=names(model$U$`u:LINE`$egv),gblup=as.numeric(model$U$`u:LINE`$egv))

data_model <- rbind(data_train,data_valid)
model <- mmer(fixed=egv~YEAR + GROUP,random=~vs(LINE,Gu=G_matrix),rcov=~vs(ds(GROUP),units),data=data_model,verbose=F,date.warning = F)
gblup_assisted <- data.frame(LINE=names(model$U$`u:LINE`$egv),gblup_assisted=as.numeric(model$U$`u:LINE`$egv))



###################
#Fit a P-BLUP model without/and with pre-existing information of the selection candidates
data_model <- rbind(data_train,data_valid)

data_model$egv[which(data_model$LINE %in% data_valid$LINE)] <- NA
model <- mmer(fixed=egv~YEAR,random=~vs(LINE,Gu=A_matrix),data=data_model,verbose=F,date.warning = F)
pblup <- data.frame(LINE=names(model$U$`u:LINE`$egv),pblup=as.numeric(model$U$`u:LINE`$egv))

data_model <- rbind(data_train,data_valid)
model <- mmer(fixed=egv~YEAR + GROUP,random=~vs(LINE,Gu=A_matrix),rcov=~vs(ds(GROUP),units),data=data_model,verbose=F,date.warning = F)
pblup_assisted <- data.frame(LINE=names(model$U$`u:LINE`$egv),pblup_assisted=as.numeric(model$U$`u:LINE`$egv))



###################
#Fit a PG-BLUP model without/and with pre-existing information of the selection candidates
data_model <- rbind(data_train,data_valid)
G_matrix <- A.mat(as.matrix(data_model[,-c(1:5)]),min.MAF=0.05)
rownames(G_matrix) <- colnames(G_matrix) <- data_model$LINE

adj <- data.frame(A=c(mean(diag(A_matrix)),mean(A_matrix)),G=c(mean(diag(G_matrix)),mean(G_matrix)))
model <- lm(A~G,data=adj)
G_matrix_mod <- model$coefficients[1] + G_matrix*model$coefficients[2]

order_correct <- data.frame(GEN=rownames(A_matrix),order_A=1:nrow(A_matrix))
order_false <- data.frame(GEN=rownames(G_matrix_mod),order_G=1:nrow(G_matrix_mod))
order_correct <- droplevels(merge(order_correct,order_false,by.x="GEN",by.y="GEN",all.x=F,all.y=F))
order_correct <- order_correct[order(order_correct$order_A,decreasing = F),]
G_matrix_mod <- G_matrix_mod[order_correct$order_G,order_correct$order_G]
#rownames(G_matrix_mod)==colnames(A_matrix)

alpha <- 0.5
beta <- 0.5
B_matrix <- alpha*G_matrix_mod + beta*A_matrix
H_matrix <- Hmatrix(A=A_matrix,G=B_matrix,tau=1,omega=1)

data_model$egv[which(data_model$LINE %in% data_valid$LINE)] <- NA
model <- mmer(fixed=egv~YEAR,random=~vs(LINE,Gu=H_matrix),data=data_model,verbose=F,date.warning = F)
pgblup <- data.frame(LINE=names(model$U$`u:LINE`$egv),pgblup=as.numeric(model$U$`u:LINE`$egv))

data_model <- rbind(data_train,data_valid)
model <- mmer(fixed=egv~YEAR + GROUP,random=~vs(LINE,Gu=H_matrix),rcov=~vs(ds(GROUP),units),data=data_model,verbose=F,date.warning = F)
pgblup_assisted <- data.frame(LINE=names(model$U$`u:LINE`$egv),pgblup_assisted=as.numeric(model$U$`u:LINE`$egv))



###################
#Fit a SSG-BLUP model without/and with pre-existing information of the selection candidates
#assuming the selection candidates have not been genotyped
data_model <- data_train
G_matrix <- A.mat(as.matrix(data_model[,-c(1:5)]),min.MAF=0.05)
rownames(G_matrix) <- colnames(G_matrix) <- data_model$LINE

A22 <- A_matrix[which(rownames(A_matrix) %in% data_model$LINE),which(colnames(A_matrix) %in% data_model$LINE)]
adj <- data.frame(A=c(mean(diag(A22)),mean(A22)),G=c(mean(diag(G_matrix)),mean(G_matrix)))
model <- lm(A~G,data=adj)
G_matrix_mod <- model$coefficients[1] + G_matrix*model$coefficients[2]

alpha <- 1
beta <- 0
B_matrix <- alpha*G_matrix_mod + beta*A22
H_matrix <- Hmatrix(A=A_matrix,G=B_matrix,tau=1,omega=1)

data_model <- rbind(data_train,data_valid)

data_model$egv[which(data_model$LINE %in% data_valid$LINE)] <- NA
model <- mmer(fixed=egv~YEAR,random=~vs(LINE,Gu=H_matrix),data=data_model,verbose=F,date.warning = F)
ssgblup <- data.frame(LINE=names(model$U$`u:LINE`$egv),ssgblup=as.numeric(model$U$`u:LINE`$egv))

data_model <- rbind(data_train,data_valid)
model <- mmer(fixed=egv~YEAR + GROUP,random=~vs(LINE,Gu=H_matrix),rcov=~vs(ds(GROUP),units),data=data_model,verbose=F,date.warning = F)
ssgblup_assisted <- data.frame(LINE=names(model$U$`u:LINE`$egv),ssgblup_assisted=as.numeric(model$U$`u:LINE`$egv))



######################################
######################################
#Validate the predictions of the various models
validation <- data_valid[,c(1,4,5)]

validation <- droplevels(merge(validation,gblup,by.x="LINE",by.y="LINE",all.x=F,all.y=F))
validation <- droplevels(merge(validation,pblup,by.x="LINE",by.y="LINE",all.x=F,all.y=F))
validation <- droplevels(merge(validation,pgblup,by.x="LINE",by.y="LINE",all.x=F,all.y=F))
validation <- droplevels(merge(validation,ssgblup,by.x="LINE",by.y="LINE",all.x=F,all.y=F))

validation <- droplevels(merge(validation,gblup_assisted,by.x="LINE",by.y="LINE",all.x=F,all.y=F))
validation <- droplevels(merge(validation,pblup_assisted,by.x="LINE",by.y="LINE",all.x=F,all.y=F))
validation <- droplevels(merge(validation,pgblup_assisted,by.x="LINE",by.y="LINE",all.x=F,all.y=F))
validation <- droplevels(merge(validation,ssgblup_assisted,by.x="LINE",by.y="LINE",all.x=F,all.y=F))

#The prediction accuarcy expressed as the correlation with the true genotypic value (tgv)
cor(validation[,-1])[,2]
#cor(validation[,-1])